/*---------------------------------------------------------------------------------

	default ARM7 core

	Copyright (C) 2005
		Michael Noland (joat)
		Jason Rogers (dovoto)
		Dave Murphy (WinterMute)

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any
	damages arising from the use of this software.

	Permission is granted to anyone to use this software for any
	purpose, including commercial applications, and to alter it and
	redistribute it freely, subject to the following restrictions:

	1.	The origin of this software must not be misrepresented; you
		must not claim that you wrote the original software. If you use
		this software in a product, an acknowledgment in the product
		documentation would be appreciated but is not required.
	2.	Altered source versions must be plainly marked as such, and
		must not be misrepresented as being the original software.
	3.	This notice may not be removed or altered from any source
		distribution.

---------------------------------------------------------------------------------*/

#include <nds.h>
#include <dswifi7.h>
#include "../../common/sipc.h"
#include "as_lib7.h"

touchPosition first;
touchPosition tempPos;

void VcountHandler() {

	static int lastbut = -1;

	uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0;

	but = REG_KEYXY;

	if (!( (but ^ lastbut) & (1<<6))) {

		tempPos = touchReadXY();

		if ( tempPos.x == 0 || tempPos.y == 0 ) {
			but |= (1 <<6);
			lastbut = but;
		} else {
			x = tempPos.x;
			y = tempPos.y;
			xpx = tempPos.px;
			ypx = tempPos.py;
			z1 = tempPos.z1;
			z2 = tempPos.z2;
		}

	} else {
		lastbut = but;
		but |= (1 <<6);
	}

	IPC->touchX			= x;
	IPC->touchY			= y;
	IPC->touchXpx		= xpx;
	IPC->touchYpx		= ypx;
	IPC->touchZ1		= z1;
	IPC->touchZ2		= z2;
	IPC->buttons		= but;


    // Check if the lid has been closed.
    if(but & BIT(7)) {
        u32 ie_save = REG_IE;
        swiChangeSoundBias(0,0x400);
        int power = readPowerManagement(PM_CONTROL_REG);
        writePowerManagement(PM_CONTROL_REG, PM_LED_BLINK);
        REG_IE = IRQ_LID;

        swiSleep(); //waits for PM (lid open) interrupt
        REG_IF = ~0;
        REG_IE = ie_save;
        writePowerManagement(PM_CONTROL_REG, power);
        swiChangeSoundBias(1,0x400);
    }
}

void ToggleBacklight()
{
    static int b = -1;
    ++b;
    b %= 4;
    writePowerManagement(4, b);
}

void VblankHandler() {
   	AS_SoundVBL(); // Update AS_Lib
    Wifi_Update(); // Update WIFI
}

void arm7SyncFunction() {
    REG_IPC_FIFO_TX = MSG_WIFI_SYNC;
}

void FIFOHandler() {
    while (!(REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY)) {
        u32 command = REG_IPC_FIFO_RX;

        switch (command) {
            case MSG_WIFI_INITIALIZE:
                while (REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY) {
                    swiWaitForVBlank();
                }

                u32 receivedValue = REG_IPC_FIFO_RX;
                Wifi_Init(receivedValue);
                Wifi_SetSyncHandler(arm7SyncFunction);
                break;
            case MSG_WIFI_SYNC:
                Wifi_Sync();
                break;
            case COMMAND7_TOGGLE_BACKLIGHT:
                ToggleBacklight();
                break;
        }
    }
}

//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------
	REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR;

	readUserSettings();
	SetYtrigger(80);
	

	//Enable sound
	powerON(POWER_SOUND);
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
   	AS_Init();

	irqInit();
	initClockIRQ();
	irqSet(IRQ_VBLANK, VblankHandler);
    irqSet(IRQ_WIFI, Wifi_Interrupt);
	irqSet(IRQ_VCOUNT, VcountHandler);
	irqSet(IRQ_FIFO_NOT_EMPTY, FIFOHandler);

	REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_RECV_IRQ;
	irqEnable(IRQ_FIFO_NOT_EMPTY | IRQ_VBLANK | IRQ_WIFI | IRQ_VCOUNT);

    do {
        while (REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY) {
  	    	AS_MP3Engine();
    	    swiWaitForVBlank();
        }
    } while(1);
}


