#include "choiceview.h"
#include "vnimage.h"
#include "vnds.h"

ChoiceView::ChoiceView(VNDS* vnds) {
    this->vnds = vnds;
    
    // Note: Large fontsizes can cause the rendering to become very slow.
    // This causes the music to crackle and stop.
    fontSize = 12;
    pad = 8;
    
    selected = 0;
    choiceL = 0;
    inited = false;    
}
ChoiceView::~ChoiceView() {
    Destroy();
}

void ChoiceView::ShowChoice(Command cmd) {
    if (IsActive()) {
        Destroy();
    }
    inited = true;

    scrollY = pad + fontSize;
    
    background = new u16[256 * 192];
    cursor = new u16[12 * 12];
    LoadImage("../../data/cursor.png", cursor, 12, 12);
    
    for (int n = 0; n < MAX_CHOICES; n++) {
        labels[n] = new char[LABEL_LEN];
    }

    backgroundDirty = true;
    selected = 0;
    for (int n = 0; n < cmd.choice.numopt; n++) {
        strncpy(labels[n], cmd.choice.options[n], LABEL_LEN-1);
        labels[LABEL_LEN-1] = '\0';
        vnds->TextReplace(labels[n], LABEL_LEN);
    }
             
    choiceL = cmd.choice.numopt;
}
void ChoiceView::Destroy() {
    if (IsActive()) {
        inited = false;

        delete[] background;
        delete[] cursor;
        for (int n = 0; n < MAX_CHOICES; n++) {
            delete[] labels[n];
        }
        
        vnds->VideoInit();
    }
}

bool ChoiceView::IsActive() {
    return inited;
}

void ChoiceView::Update() {
    int oldSelected = selected;
    //int held = keysHeld();
    int down = keysDown();

    if (down & KEY_DOWN) {
        selected++;
        if (selected >= choiceL) selected -= choiceL;
    }
    if (down & KEY_UP) {
        selected--;
        if (selected < 0) selected += choiceL;
    }
    if (down & KEY_RIGHT) {
        selected = choiceL - 1;
    }
    if (down & KEY_LEFT) {
        selected = 0;
    }
    
    /*
    if (held & KEY_TOUCH) {
        touchPosition touchXY = touchReadXY();
        
        int l = 0;
        selected = 0;
        for (int n = 0; n < choiceL; n++) {        
             l += text->PrintStringWrapLines(labels[n]);
             if (touchXY.py > l) {
                 selected += 1;
             }
        }
    }*/

    backgroundDirty |= selected != oldSelected;

    if (down & (KEY_B | KEY_A)) {
        vnds->SelectChoice(selected);
    }
}

void ChoiceView::DrawBackground(Text* text) {
    memset(background, 0, 256 * 192 * sizeof(u16));
    
    text->SetScreen(background);
    text->SetPixelSize(fontSize);
    text->SetMargins(20 + pad, pad, pad, pad);
    text->SetPen(20 + pad, pad);

    int lines0 = 0;
    int lines1 = 0;
    for (int n = 0; n < choiceL; n++) {        
        int l = text->PrintStringWrapLines(labels[n]);

        if (n <= selected) {
            if (n < selected) {
                lines0 += l;
            }
            lines1 += l;
        }
    }

    if (selected >= 0 && selected < choiceL) {
        int top    =       pad - lines0 * (text->GetHeight() + LINESPACING);
        int bottom = 192 - pad - lines1 * (text->GetHeight() + LINESPACING);
        
        scrollY = MAX(top, MIN(bottom, scrollY));
    }

    int dy = scrollY;
    for (int n = 0; n < choiceL; n++) {
        if (n == selected) {
            int offsetY = dy - 10;
            if (offsetY >= 0 && offsetY+12 < 192) {
                for (int y = 0; y < 12; y++) {
                    memcpy(background + (offsetY+y) * 256 + 4, cursor + y * 12, 12 * sizeof(u16));
                }
            }
        }
        text->SetPen(18, dy);
        text->PrintStringWrap(labels[n]);
        text->PrintNewLine();
        dy = text->GetPenY();
    }
}

void ChoiceView::Draw(u16* screen, Text* text) {
    Update();
    if (!IsActive()) {
        return;
    }
    
    if (backgroundDirty) {
        DrawBackground(text);
        backgroundDirty = false;
    }
   	swiCopy(background, screen, 256 * 192 / 2 | COPY_MODE_WORD);
}
