#include "filehandle.h"
#include "vncommon.h"

FileHandle* fhOpen(char* filename) {
    return fhOpen(NULL, filename);
}
FileHandle* fhOpen(Archive* archive, char* filename) {
    //Try regular file first
    FILE* file = fopen(filename, "rb");
    if (file) {
        DefaultFileHandle* fh = new DefaultFileHandle(file, filename);
        return fh;
    }
    
    //Then try archive
    if (archive) {
        FileHandle* fh = archive->OpenFile(filename);
        if (fh) {
            return fh;
        }
    }
    
    return NULL;
}
void fhClose(FileHandle* fh) {
    if (fh) {
        delete fh;
    }
}
void fhSeek(FileHandle* fh, u32 offset, int seekType) {
    fh->Seek(offset, seekType);
}
int fhTell(FileHandle* fh) {
    return fh->Tell();
}
int fhRead(void* out, u32 len, FileHandle* fh) {
    return fh->Read(out, len);
}
bool fhReadFully(void* out, FileHandle* fh) {    
    return fh->ReadFully(out);
}

//=== FileHandle methods =======================================================
void FileHandle::Seek(u32 offset, int seekType) {
    int base = currentOffset;    
    if (seekType == SEEK_SET) {
        base = startOffset;
    } else if (seekType == SEEK_END) {
        base = startOffset + length;
    }
    currentOffset = base + offset;    
}
int FileHandle::Tell() {
    return currentOffset;
}
bool FileHandle::ReadFully(void* out) {
    Seek(0, SEEK_SET);
    return (u32)Read(out, length) == length;
}

//=== DefaultFileHandle methods ================================================
int DefaultFileHandle::Read(void* out, u32 len) {
    Seek(currentOffset - startOffset, SEEK_SET);
    int read = fread(out, 1, len, file);
    if (read >= 0) {
        currentOffset += read;
    }
    return read;
}

//=== ZipEntryHandle methods ===================================================

int ArchiveEntryHandle::Read(void* out, u32 len) {
    int read = archive->Read(out, currentOffset, MIN(len, length - (currentOffset - startOffset)));
    if (read >= 0) {
        currentOffset += read;
    }
    return read;
}
