#ifndef _FILEHANDLE_H_
#define _FILEHANDLE_H_

#include <nds.h>
#include <string.h>
#include "dsunzip.h"

class FileHandle {
    
    public:
        char filename[256];
        u32 startOffset;
        u32 currentOffset;
        u32 length;
        
        FileHandle(char* filename) {
            strcpy(this->filename, filename);
        }
        virtual ~FileHandle() {
        }
        
        virtual void Seek(u32 offset, int seekType);
        virtual int Tell();
        virtual int Read(void* out, u32 len) = 0; //Pure virtual
        virtual bool ReadFully(void* out);
};

class DefaultFileHandle : public FileHandle {
    
    private:
        FILE* file;
    
    public:
        DefaultFileHandle(FILE* file, char* filename)
            : FileHandle(filename)
        {
            this->file = file;
            this->startOffset = 0;
            this->currentOffset = 0;
            
            fseek(file, 0, SEEK_END);
            this->length = ftell(file);
            fseek(file, 0, SEEK_SET);
        }
        virtual ~DefaultFileHandle() {
            fclose(file);
        }
        
        virtual int Read(void* out, u32 len);
};

class ArchiveEntryHandle : public FileHandle {
    
    private:
        Archive* archive;
        u32 entryIndex;
    
    public:
        ArchiveEntryHandle(Archive* archive, char* filename, u32 entryIndex)
            : FileHandle(filename)
        {
            this->archive = archive;
            this->entryIndex = entryIndex;
            
            this->startOffset = archive->offsets[entryIndex];
            this->currentOffset = startOffset;
            this->length = archive->filesizes[entryIndex];
        }
        virtual ~ArchiveEntryHandle() {
        }
        
        virtual int Read(void* out, u32 len);
};

FileHandle* fhOpen(char* filename);
FileHandle* fhOpen(Archive* archive, char* filename);
void fhClose(FileHandle* fh);
void fhSeek(FileHandle* fh, int offset, int seekType);
int fhTell(FileHandle* fh);
int fhRead(void* out, int len, FileHandle* fh);
bool fhReadFully(void* out, FileHandle* fh);

#endif
