#ifndef _SAVEMENU_H_
#define _SAVEMENU_H_

#include <nds.h>

#include "vncommon.h"
#include "util_3d.h"
#include "text.h"
#include "xmlParser.h"
#include "sound.h"

class VNDS;

#define SAVE_SLOTS 18
#define SAVE_IMG_PAD 8
#define SAVE_IMG_W 64
#define SAVE_IMG_H 48

const int SLOTS_PER_PAGE = MIN(SAVE_SLOTS, 6);

class SaveMenu {
    private:        
        VNDS* vnds;
        bool inited;
        
        u16* videobuf_main;
        u16* videobuf_sub;
        u16 background[256 * 192];
        
        u16 backgroundImage[256 * 192];
        u16 textureImage[128 * 128];
        u16 emptySlotImage[SAVE_IMG_W * SAVE_IMG_H];
        u16 defaultSlotImage[SAVE_IMG_W * SAVE_IMG_H];

        int page;
        int pageMin;
        int pageMax;
        int selected;

        bool backgroundDirty;
        int animation;
        int frame;
        int slideDirection;

        bool saveSlotExists[SAVE_SLOTS];
        char saveSlotLabels[SAVE_SLOTS][32];
        u16* saveSlotImages[SAVE_SLOTS];
        int saveSlotTextures[SLOTS_PER_PAGE];

        Button* buttons[5];
        Button* popupButtons[2];
        int buttonTexture;
        int buttonPressed;
        int buttonPressedFrame;
        int popupMenu;
        char popupMessage[128];

        void DeleteButtons();
        void OnButtonPress(int n);
        void OnPopupButtonPress(int n);
        void ReloadTextures();
        void Update();
        void DrawBackground(Text* text);
        void SetPage(int p);
        
        //char soundbuf[1024*12];
        //void PlaySound(char*);

    public:
        SaveMenu(VNDS* vnds);
        ~SaveMenu();
        
        void Init();
        void Destroy();
        void Reload();
        void Draw(Text* text);
        bool IsActive();
};

#endif
