#include "script.h"

#include <ctype.h>

const int SCRIPT_SIZE = 1024*4; // 4k blocks seems fine
const int BUF_L = 1024;

Script::Script(Archive* archive, char* filename) {
    strcpy(path, filename);
    script = NULL;

    fh = fhOpen(archive, path);
    if (!fh) {
        //This is very bad
        return;
    }
    
    filesize = fh->length;
    
    rindex = 0;
    script = new char[SCRIPT_SIZE];
    
    fh->Read(script, SCRIPT_SIZE);
    //fh->ReadFully(script);
    //script[filesize] = '\0';
    //fhClose(fh);

    scrindex = 0;
}

Script::~Script() {
    if (script) {
        delete[] script;
        script = NULL;
    }
    fhClose(fh);
}

const char* Script::GetScript()
{
    return path;
}

Command Script::NextCommand()
{   
    while (script[rindex] == ' ' || script[rindex] == '\t') {
        rindex++;
    }
    
    char* buf = new char[BUF_L];
    int l;
    //int s;
    for (l = 0; l < BUF_L - 1; ++l, ++rindex) {
        if (rindex > SCRIPT_SIZE) {
            rindex = 0;
            //s = 0;
            fh->Read(script, SCRIPT_SIZE);
        }
        buf[l] = script[rindex];
        
        if (script[rindex] == '\n' || script[rindex] == '\0') {
            break;
        }
    }    
    buf[l] = '\0';
    scrindex += l;
    if (script[rindex] == '\n') rindex++;
    
    char cmd[16];
    memcpy(cmd, buf, 15);
    cmd[15] = '\0';    
    for (int n = 0; n < 15; n++) {
        if (buf[n] == ' ') {
            cmd[n] = '\0';
            break;
        }
    }
    
    Command c;
    memset(&c, 0, sizeof(c));
        
    if (cmd[0] == '#') {
        c.id = SKIP;
    }
    else if (strcmp(cmd, "bgload") == 0) {
        c.id = BGLOAD;
        //sscanf(buf, "%15s %127s %63s", cmd, c.bgload.path, c.bgload.trans);
        sscanf(buf, "%15s %127s", cmd, c.bgload.path);
    }
    else if (strcmp(cmd, "setimg") == 0) {
        c.id = SETIMG;
        sscanf(buf, "%15s %127s %d %d", cmd, c.setimg.path, &c.setimg.x, &c.setimg.y);
    }
    else if (strcmp(cmd, "sound") == 0) {
        c.id = SOUND;
        c.sound.times = 1;
        sscanf(buf, "%15s %127s %d", cmd, c.sound.path, &c.sound.times);
    }
    else if (strcmp(cmd, "music") == 0) {
        c.id = MUSIC;
        sscanf(buf, "%15s %127s", cmd, c.music.path);
    }
    else if (strcmp(cmd, "text") == 0) {
        //Check if the character after text is a ' '. If this is not the case, it may
        //well be a multibyte char. If this is the case, you must not skip the first byte.        
        int offset = 4;
        if (buf[4] == ' ') offset++;
        
        c.id = TEXT;
        memcpy(c.text.text, buf + offset, l-offset);
        c.text.text[l-offset] = '\0';
    }
    else if (strcmp(cmd, "choice") == 0) {
        c.id = CHOICE;

        char* temp = buf + 6;
        if (buf[6] == ' ') temp++;
        
        int opt = 0;
        int ind = 0;
        for(int i = 0; temp[i] != '\0'; i++) {
            if (temp[i] == '|'){
                ++opt;
                ind = 0;
                continue;
            }
            c.choice.options[opt][ind++] = temp[i];
        }
        
        c.choice.numopt = opt + 1;
    }
    else if (strcmp(cmd, "setvar") == 0) {
        c.id = SETVAR;
        //char tmp[128];
        sscanf(buf, "%15s %63s %c", cmd, c.setvar.name, &c.setvar.op);
        
        int i = 0;
        int d = 0;
        while (true) { // kill stuff till we get where we want
            if (buf[i] == ' ')
                ++d;
            if (d == 3)
                break;
            ++i;
        }
        ++i;
        strncpy(c.setvar.value, buf+i, 128);
        
        //if ('0' <= tmp[0] and tmp[0] <= '9') {
        /*if (isdigit(tmp[0])) {
            c.setvar.value.type = NUMBER;
            sscanf(tmp, "%d", &c.setvar.value.number);
        }
        else {
            c.setvar.value.type = STRING;
            strcpy(c.setvar.value.string, tmp);
        }*/
    }
    else if (strcmp(cmd, "gsetvar") == 0) {
        c.id = GSETVAR;
        //char tmp[128];
        sscanf(buf, "%15s %63s %c", cmd, c.setvar.name, &c.setvar.op);
        
        int i = 0;
        int d = 0;
        while (true) { // kill stuff till we get where we want
            if (buf[i] == ' ')
                ++d;
            if (d == 3)
                break;
            ++i;
        }
        ++i;
        strncpy(c.setvar.value, buf+i, 128);
        
        /*if (isdigit(tmp[0])) {
            c.setvar.value.type = NUMBER;
            sscanf(tmp, "%d", &c.setvar.value.number);
        }
        else {
            c.setvar.value.type = STRING;
            strcpy(c.setvar.value.string, tmp);
        }*/
    }
    else if (strcmp(cmd, "if") == 0) {
        c.id = IF;
        sscanf(buf, "%15s %63s %3s", cmd, c.vif.name, c.vif.op);
        
        int i = 0;
        int d = 0;
        while (true) { // kill stuff till we get where we want
            if (buf[i] == ' ')
                ++d;
            if (d == 3)
                break;
            ++i;
        }
        ++i;
        strncpy(c.vif.name2, buf+i, 127);
        c.vif.name2[127] = '\0';
    }
    else if (strcmp(cmd, "fi") == 0) {
        c.id = FI;
    }
    else if (strcmp(cmd, "jump") == 0) {
        c.id = JUMP;
        sscanf(buf, "%15s %127s", cmd, c.jump.path);
    }
    else if (strcmp(cmd, "delay") == 0) {
        c.id = DELAY;
        sscanf(buf, "%15s %i", cmd, &c.delay.time);
    }
    else if (strcmp(cmd, "random") == 0) {
        c.id = RANDOM;
        sscanf(buf, "%15s %63s %d %d", cmd, c.random.name, &c.random.low, &c.random.high);
    }
    else {
        c.id = SKIP;
    }
    
    delete[] buf;
    return c;
}
