#include "sound.h"

#include <stdlib.h>

#include "vncommon.h"
#include "aac.h"
#include "as_lib9.h"

//Reading music from Zip-Files requires modification of AS_Lib
Music::Music(Archive* archive, char* p) {
    playing = false;

    char path[256];
    sprintf(path, "sound/%s", p);

    int L = strlen(p);
    if (L > 3 && (p[L-3] == 'W' || p[L-3] == 'w' || p[L-3] == 'o')) {
        //If file-ext indicates ADPCM (or .ogg)
        //exit, since it can't be played

        return;
    }

    AS_SetMP3Loop(TRUE);
    AS_MP3StreamPlay(path);

    playing = true;
}

Music::~Music() {
    if (playing) {
        playing = false;

        AS_MP3Stop();
    }
}

Sound::Sound(Archive* archive, char* p, int t) {
    isAAC = false;
    sfxbuf = NULL;
    channel = -1;
    if (t == 0) {
        return;
    }

    //Read file from disk
    char path[256];
    sprintf(path, "sound/%s", p);
    count = 0;
    times = t;

    //If the file is .aac, decode it
    int L = strlen(p);
    if (L > 3 && (p[L-3] == 'a' && p[L-2] == 'a' && p[L-1] == 'c')) {
        isAAC = true;
        aacPlayer->PlaySound(archive, path);
    } else {
        //By default, assume ADPCM
        FileHandle* fileHandle = fhOpen(archive, path);
        if (!fileHandle) {
            return;
        }
    
        #define FILE_READ_SIZE (256 * 1024)

        int sfxbufL;
        sfxbufL = MIN(FILE_READ_SIZE, fileHandle->length);
        sfxbuf = new u8[sfxbufL];
        
        sfxbufL = fileHandle->Read(sfxbuf, sfxbufL);
        sfxbufL = MAX(0, sfxbufL);
        
        channel = AS_SoundDefaultPlay((u8*)sfxbuf, sfxbufL, 127, 64, (t < 0), 0);

        fhClose(fileHandle);
    }
}

Sound::~Sound() {
    if (isAAC) {
        aacPlayer->StopSound();
    } else {    
        if (channel >= 0) {
            AS_SoundStop(channel);
    		channel = -1;
        }
        
        if (sfxbuf) {
            delete[] sfxbuf;
        }
    }
}
