#include "util_3d.h"

Button::Button(int texture, Rect rect, Rect uv) {
    this->texture = texture;
    this->rect = rect;
    this->uv = uv;
}
Button::~Button() {
}

bool Button::Contains(int x, int y) {
    return x >= rect.x && x < rect.x + rect.w && y >= rect.y && y < rect.y + rect.h;
}
void Button::Draw(bool selected) {
    int s = (selected ? 2 : 0);
    drawQuad(texture, rect.x, rect.y + s, rect.w, rect.h, uv.x, uv.y, uv.x + uv.w, uv.y + uv.h);
}

void glTranslate(int x, int y) {
    glTranslate3f32(inttof32(x), inttof32(y), 0);
}

void drawQuad(int texture, int x, int y, int w, int h) {
    drawQuad(texture, x, y, w, h, 0, 0, w, h);
}

void drawQuad(int texture, int x, int y, int w, int h, int u0, int v0, int u1, int v1) {
    glPushMatrix();
    glTranslate(x, y);

    float vw = w * INVSCALE;
    float vh = h * INVSCALE;

    glScalef(SCALE, SCALE, 1);
	glBindTexture(GL_TEXTURE_2D, texture);
	glBegin(GL_QUAD);
		glNormal(NORMAL_PACK(0, 0, inttov10(1)));
		glTexCoord2t16(inttot16(u0), inttot16(v0));
        glVertex3f(0, 0, 0);
		glTexCoord2t16(inttot16(u1), inttot16(v0));
        glVertex3f(vw, 0, 0);
		glTexCoord2t16(inttot16(u1), inttot16(v1));
        glVertex3f(vw, vh, 0);
		glTexCoord2t16(inttot16(u0), inttot16(v1));
        glVertex3f(0, vh, 0);
	glEnd();

	glPopMatrix(1);
}

void init3D() {
	glInit();
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	//glEnable(GL_ANTIALIAS);

	//Setup the rear plane
	glClearColor(0, 0, 0, 0); // BG must be opaque for AA to work
	glClearPolyID(63); // BG must have a unique polygon ID for AA to work
	glClearDepth(0x7FFF);

	//Set our viewport to be the same size as the screen
	glViewport(0, 0, 255, 191);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//gluPerspective(70, 256.0 / 192.0, 0.1, 100);
	glOrtho(0, 255 * INVSCALE, 191 * INVSCALE, 0, 0.1, 100);

	glMaterialf(GL_AMBIENT, RGB15(16,16,16));
	glMaterialf(GL_DIFFUSE, RGB15(16,16,16));
	glMaterialf(GL_SPECULAR, BIT(15) | RGB15(8,8,8));
	glMaterialf(GL_EMISSION, RGB15(16,16,16));
    glMaterialShinyness();
	glLight(0, RGB15(31,31,31), 0, floattov10(-1.0), 0);	
}
