#ifndef _VNDS_H_
#define _VNDS_H_

#include <nds.h>
#include <string>
#include <map>

#include "dsunzip.h"
#include "script.h"
#include "text.h"
#include "sound.h"
#include "savemenu.h"
#include "choiceview.h"
#include "xmlParser.h"

const int TOUCHSCROLLDELAY = 3;
const int MAX_SPRITES = 12;

struct SaveNumber {
    char var[64];
    int value;
};
struct SaveString {
    char var[64];
    char value[128];
};

struct Sprite {
    char path[128];
    int x, y;
};

const int GAMELOG = 512;
const int GAMESTR = 256;

class VNDS {
    private:
        u16* videobuf_main;
        u16* videobuf_sub;        
        u16 screen_top[256*192];
        u16 screen_btm[256*192];
        
        Archive* foregroundArc;
        Archive* backgroundArc;
        Archive* scriptArc;
        Archive* soundArc;

        int delay;
        int ticks;
        bool run;

        int tcursor;
        int tend;
        char* gametext[GAMELOG];
        char currentfile[128];
        
        Script* script;
        Music* music;
        Sound* sound;
        Text* text;

        std::map<std::string, Variable> globals;
        std::map<std::string, Variable> variables;
            
        SaveMenu* saveMenu;
        ChoiceView* choiceView;
        int textskip;
        bool waitfortext;
        
        int GetNextCommand();                
        void NewScript(char*);
        
        int scount;
        Sprite sprites[MAX_SPRITES];
        char bgpath[128];
        char musicpath[128];
        
        void GlobalLoad();
        void GlobalSave();
        
        void ScreenShot();
        void Skip(int maxSkip, bool loadGameMode);
        void StopSound();
        void StopMusic();

        void SetBackground(Command);
        void SetImage(Command);
        void PlaySound(Command);
        void PlayMusic(Command);
        void If(Command);
        void SetVarRandom(Command);
        void SetVar(Command);
        void LoadScript(Command);
    
        void HandleInput();
    
    public:
        VNDS();
        ~VNDS();
        
        void VideoInit();
        void ReturnToMenu();
        void Save(int slot);
        void Load(int slot);
        void Run();

        void TextReplace(char*, int);
        void TodayDate(char*);
        void ShowChoice(Command);
        void SelectChoice(int);
        
        void AppendText(Command);
        void DisplayText();
};

#endif /* _VNDS_H_ */
