#include "vnwifi.h"

#include <nds.h>
#include <dswifi9.h>
#include <stdio.h>
#include "../common/sipc.h"

#define WIFI_PING 50

void fifoHandlerFunction(void);
void timerFunction(void);
void arm7SyncFunction(void);

void initWifi() {
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR;

    TIMER3_CR = 0; // Disable timer3 while we set up its interval
    irqInit();

    //Init IRQ
    irqSet(IRQ_TIMER3, timerFunction);
    irqEnable(IRQ_TIMER3);

    irqSet(IRQ_FIFO_NOT_EMPTY, fifoHandlerFunction);
    irqEnable(IRQ_FIFO_NOT_EMPTY);

  	irqEnable(IRQ_VBLANK);

    // Now the FIFO must be re-configured
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_RECV_IRQ;

    //Set synchandler
    Wifi_SetSyncHandler(arm7SyncFunction);

    //Send init message
    u32 wifi_passdata = Wifi_Init(WIFIINIT_OPTION_USELED);
    REG_IPC_FIFO_TX = MSG_WIFI_INITIALIZE;
    REG_IPC_FIFO_TX = wifi_passdata;

    //Set timer
    TIMER3_DATA = TIMER_FREQ_256(WIFI_PING);
    TIMER3_CR = TIMER_ENABLE | TIMER_IRQ_REQ | TIMER_DIV_256;

    // Now we must wait for the Wifi hardware to finish initialization..
    while (Wifi_CheckInit() == 0) {
        swiWaitForVBlank();
    }
}

void startWifi() {
	Wifi_EnableWifi();
    Wifi_AutoConnect();
}
void stopWifi() {
    Wifi_DisableWifi();
}

void connectWifi() {    
    int status = Wifi_AssocStatus();
    switch (status) {
        case ASSOCSTATUS_CANNOTCONNECT:
            printf("Wifi error: Unable to connect to AP.\n");
            break;
        case ASSOCSTATUS_ASSOCIATED:
            printf("Wifi connection success. Associated to an AP.\n");
            break;
        case ASSOCSTATUS_DISCONNECTED:
            printf("Wifi error: not trying to connect???.\n");
            break;            
        case ASSOCSTATUS_SEARCHING:
            printf("Wifi error: searching???.\n");
            break;            
        default:
            printf("Unknown Status: %d\n", status);
    }
}

void fifoHandlerFunction() {
    u32 message = REG_IPC_FIFO_RX;

    switch (message) {
        case MSG_WIFI_SYNC:
            Wifi_Sync();
            break;
    }
}

void timerFunction() {
	Wifi_Timer(WIFI_PING);
}
void arm7SyncFunction() {
    REG_IPC_FIFO_TX = MSG_WIFI_SYNC;
}
       
WifiMenu::WifiMenu() {
    consoleDemoInit();
    printf("Starting Wifi...\n");
    startWifi();
    printf("Thundercats are go!!!\n");
}
WifiMenu::~WifiMenu() {
    stopWifi();
}

void WifiMenu::Draw() {
    printf("Connectan gaems\n");

    connectWifi();
    for (int n = 0; n < 120; n++) {
        swiWaitForVBlank();
    }
}
